# SPDX-FileCopyrightText: 2023-present Massimiliano Pippi <mpippi@gmail.com>
#
# SPDX-License-Identifier: MIT
from .directory import DirectoryPromptRegistry
from .file import FilePromptRegistry

__all__ = ("FilePromptRegistry", "DirectoryPromptRegistry")
