{% chat role="system" %}
You are a helpful assistant.
{% endchat %}

{% chat role="user" %}
{{ "Hello, <bold>how are you?</bold>" | cache_control("ephemeral") }}
{% endchat %}

{% chat role="system" %}
I'm doing well, thank you! How can I assist you today?
{% endchat %}

{% chat role="user" %}
Can you explain quantum computing?
{% endchat %}

Some random text.
